/*
        Copyright 2006 Bianca van Schaik

    This file is part of the MoonPhase widget.

    MoonPhase is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MoonPhase is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MoonPhase; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

var last_calculation = 0;
var IMAGE_DIR = "images";

function daysTo(a, b) {
	var ms_per_day = 86400000;
	return Math.round((b - a) / ms_per_day);
}

function setMoon() {
	var now = new Date();
	// Adjust for timezone offset
	var tz = now.getTimezoneOffset();
	now.setTime(now.getTime() + 60 * 1000 * tz);
	
	var now_epoch = now.getTime();
	
	// Don't re-calculate if the last calculation was less than 10 minutes ago
	if (now_epoch - last_calculation < 600000) {
		return;
	}
	
	var jde = moonphasebylunation(0, 0);
	// Approximate lun, saves 1000+ loops
	var lun = Math.floor((DatetoJD(now) - jde) / 29.53);
	var last_new, next_new;

	do {
		jde = moonphasebylunation(lun, 0);
		last_new = next_new;
		next_new = JDtoDate(jde, 0);
		lun++;
	} while (next_new < now_epoch);

    lun -= 2;

	var first_quarter = JDtoDate( moonphasebylunation( lun, 1 ), 0);
	var full_moon = JDtoDate( moonphasebylunation( lun, 2 ), 0 );
	var last_quarter = JDtoDate( moonphasebylunation( lun, 3 ), 0 );

/*
	var d = new Date();
	d.setTime(last_new);
	log("Last new moon: "+d.toString());
	d.setTime(first_quarter);
	log("First quarter: "+d.toString());
	d.setTime(full_moon);
	log("Full moon: "+d.toString());
	d.setTime(last_quarter);
	log("Last quarter: "+d.toString());
	d.setTime(next_new);
	log("Next new moon: "+d.toString());
*/

	var counter;
	var phase;
	var extra = "";
	
	var days_to_ln = daysTo(now_epoch, last_new);
	var days_to_fq = daysTo(now_epoch, first_quarter);
	if (days_to_ln == 0) {
		counter = 0;
		phase = "New Moon";
	} else if (days_to_fq == 0) { // is first quarter
		counter = 7;
		phase = "First Quarter";
	} else if (days_to_fq > 0) { // between new moon and first quarter
		counter = -days_to_ln;
		phase = "Waxing Crescent";
		extra = (days_to_ln == -1) ? "Yesterday was New Moon" :
			"" + (-days_to_ln) + " days since New Moon";
	} else {
		var days_to_fm = daysTo(now_epoch, full_moon);
		if (days_to_fm == 0) { // is full moon
			counter = 16;
			phase = "Full Moon";
		} else if (days_to_fm > 0) { // between first quarter and new moon
			counter = 16 - days_to_fm;
			phase = "Waxing Gibbous";
			extra = (days_to_fm == 1) ? "Tomorrow is Full Moon" :
				"" + (days_to_fm) + " days to Full Moon";
		} else {
			var days_to_lq = daysTo(now_epoch, last_quarter);
			if (days_to_lq == 0) { // is last quarter
				counter = 24;
				phase = "Last Quarter";
			} else if (days_to_lq > 0) { // between full moon and last quarter
				counter = 16 - days_to_fm;
				phase = "Waning Gibbous";
				extra = (days_to_fm == -1) ? "Yesterday was Full Moon" :
					"" + (-days_to_fm) + " days since Full Moon";
			} else {
				var days_to_nm = daysTo(now_epoch, next_new);
				if (days_to_nm == 0) { // new moon
					counter = 0;
					phase = "New Moon";
				} else { // between last quarter and new moon
					counter = 32 - days_to_nm;
					phase = "Waning Crescent";
					extra = (days_to_nm == 1) ? "Tomorrow is New Moon" :
						"" + (days_to_nm) + " days to New Moon";
				}
			}
		}
	}
/*
	log(phase);
	log(extra);
	log("Days to last new: "+daysTo(now_epoch, last_new));
	log("Days to first quarter: "+daysTo(now_epoch, first_quarter));
	log("Days to full moon: "+daysTo(now_epoch, full_moon));
*/
	// Update widget
	var mp = document.getElementById('moon-phase');
	mp.innerHTML = phase;
	var me = document.getElementById('moon-extra');
	me.innerHTML = extra;
	var mi = document.getElementById('moon-image');
	mi.src = IMAGE_DIR + "/moon"+(counter)+".png";
	
	last_calculation = now_epoch;
}

function setup() {
	setMoon();
	
	if (window.widget) {
		widget.onshow = setMoon;
	}
}

// debug function
function log(msg) {
	var x = document.getElementById('status');
	x.innerHTML += '<div>'+msg+'</div>';
}

window.onload = setup;
